using CLU.Helpers;

using CommonBehaviors.Actions;

using TreeSharp;

namespace CLU.Classes.Warlock
{
    using global::CLU.GUI;

    using global::CLU.Lists;

    class Demonology : RotationBase
    {
        public override string Name
        {
            get { return "Demonology Warlock"; }
        }

        public override string KeySpell
        {
            get { return "Summon Felguard"; }
        }

        // I want to keep moving at melee range while morph is available
        // note that this info is used only if you enable moving/facing in the CC settings.
        public override float CombatMaxDistance
        {
            get { return (Spell.CanCast("Metamorphosis", Me) || Buff.PlayerHasBuff("Metamorphosis")) ? 10f : 35f; }
        }

        public override float CombatMinDistance
        {
            get { return 30f; }
        }

        // adding some help about cooldown management
        public override string Help
        {
            get
            {
                return "\n" +
                       "----------------------------------------------------------------------\n" +
                       "This CC will:\n" +
                       "1. Soulshatter, Interupts with Axe Toss, Soul Harvest < 2 shards out of combat\n" +
                       "2. AutomaticCooldowns has: \n" +
                       "==> UseTrinkets \n" +
                       "==> UseRacials \n" +
                       "==> UseEngineerGloves \n" +
                       "==> Demon Soul & Summon Doomguard & Metamorphosis & Curse of the Elements & Lifeblood\n" +
                       "==> Felgaurd to Felhunter Swap\n" +
                       "3. AoE with Hellfire and Felstorm and Shadowflame\n" +
                       "4. Best Suited for end game raiding\n" +
                       "Ensure you're at least at 10yards from your target to maximize your dps, even during burst phase if possible.\n" +
                       "NOTE: PvP uses single target rotation - It's not designed for PvP use. \n" +
                       "Credits to cowdude\n" +
                       "----------------------------------------------------------------------\n";
            }
        }



        public override Composite SingleRotation
        {
            get
            {
                return new PrioritySelector(
                        // Pause Rotation
                        new Decorator(ret => SettingsFile.Instance.AutoManagePauseRotation, new ActionAlwaysSucceed()),

                        // Performance Timer (True = Enabled) returns Runstatus.Failure
                        // Spell.TreePerformance(true),

                        // For DS Encounters.
                        EncounterSpecific.ExtraActionButton(SettingsFile.Instance.AutoExtraActionButton),

                        // Performance Timer (True = Enabled) returns Runstatus.Failure
                        // Spell.TreePerformance(true),

                        new Decorator(
                            ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget),
                                new PrioritySelector(
                                        Item.UseTrinkets(),
                                        Spells.UseRacials(),
                                        Buff.CastBuff("Lifeblood", ret => true, "Lifeblood"),
                                        Item.UseEngineerGloves())),
                        
                        // START Interupts & Spell casts
                        new Decorator(ret => Me.GotAlivePet && PetManager.CurrentPetType == PetManager.PetType.Felguard,
                            new PrioritySelector(
                                Pets.CastPetSpell("Axe Toss", ret => Me.CurrentTarget != null && Me.CurrentTarget.IsCasting && Me.CurrentTarget.CanInterruptCurrentSpellCast && Pets.CanCastPetSpell("Axe Toss"), "Axe Toss")
                                )
                            ),

                        // lets get our pet back
                        new Decorator(ret => !Me.GotAlivePet,
                            new PrioritySelector(
                                Spells.CastSelfSpell("Soulburn", ret => !Buff.UnitHasHasteBuff(Me), "Soulburn for Pet"),
                                Pets.CastPetSummonSpell("Summon Felhunter", ret => !Me.GotAlivePet, "Felhunter (lets get our pet back)")
                                )
                            ),

                        // Threat
                        Buff.CastBuff("Soulshatter", ret => Me.CurrentTarget != null && Me.GotTarget && Me.CurrentTarget.ThreatInfo.RawPercent > 90 && !Spell.PlayerIsChanneling, "[High Threat] Soulshatter - Stupid Tank"),
                        
                        // Cooldowns
                        new Decorator(
                            ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget),
                                new PrioritySelector(
                                    Buff.CastBuff("Metamorphosis",              ret => true, "Metamorphosis"),                                    
                                    Spells.CastSelfSpell("Demon Soul",          ret => true, "Demon Soul"),
                                    Spells.CastSelfSpell("Summon Doomguard",    ret => true, "Doomguard"),
                                    Pets.CastPetSpell("Felstorm", ret => Me.CurrentTarget != null && Me.Pet.Location.Distance(Me.CurrentTarget.Location) < Spell.MeleeRange && PetManager.CurrentPetType == PetManager.PetType.Felguard, "Felstorm(Special Ability)"),
                                    Items.RunMacroText("/petattack", ret => !Me.Pet.IsAutoAttacking && Me.CurrentTarget != null, "Pet attack"),
                                    // Make sure our Felhunter did Axe Toss, before calling new pet -> Axe Toss needs to be on Cooldown
                                    Spells.CastSelfSpell("Soulburn",            ret => !Pets.CanCastPetSpell("Axe Toss") && !Pets.PetHasBuff("Felstorm"), "Soulburn to raise Felhunter"),
                                    // Call pet - regardless if we have Soulburn or not, we need this pet!
                                    Pets.CastPetSummonSpell("Summon Felhunter", ret => !Pets.PetHasBuff("Felstorm") && PetManager.CurrentPetType == PetManager.PetType.Felguard && !Pets.CanCastPetSpell("Felstorm"), "Felhunter (after axe toss)"),
                                    Items.RunMacroText("/petattack", ret => !Me.Pet.IsAutoAttacking && Me.CurrentTarget!=null, "Pet attack after Pet Swap")
                                    )),
                        Spells.CastSelfSpell("Soulburn",                ret => !Buff.UnitHasHasteBuff(Me) && !Pets.PetHasBuff("Felstorm"), "Soulburn"),
                        Spells.CastSpell("Soul Fire",                   ret => Buff.PlayerHasBuff("Soulburn"), "Soul Fire with soulburn"),
                        
                        // AoE
                        Pets.CastPetSpell("Felstorm", ret => Me.CurrentTarget != null && PetManager.CurrentPetType == PetManager.PetType.Felguard && Me.Pet.Location.Distance(Me.CurrentTarget.Location) < Spell.MeleeRange && !Pets.PetHasBuff("Felstorm") && Units.CountEnnemiesInRange(Me.CurrentTarget.Location, 12) > 0 && (Me.CurrentTarget.CurrentHealth > 310000 || Me.CurrentTarget.MaxHealth == 1), "Felstorm(Special Ability)"),
                        Spells.ChannelAreaSpell("Hellfire", 11.0, false, 4, 0.0, 0.0, ret => !BossList.IgnoreAoE.Contains(Unit.CurrentTargetEntry), "Hellfire"),
                        
                        // Main Rotation
                        Buff.CastDebuff("Curse of the Elements",        ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Me.CurrentTarget.HealthPercent > 70 && !Buff.UnitHasMagicVulnerabilityDeBuffs(Me.CurrentTarget), "Curse of the Elements"),
                        Buff.CastDebuff("Immolate",                     ret => true, "Immolate"),
                        Buff.CastDebuff("Bane of Doom",                 ret => Me.CurrentTarget != null && Units.IsTargetWorthy(Me.CurrentTarget) && Unit.TimeToDeath(Me.CurrentTarget) > 60 && !Buff.UnitsHasMyBuff("Bane of Doom"), "Bane of Doom TTL=" + Unit.TimeToDeath(Me.CurrentTarget)),
                        Buff.CastDebuff("Bane of Agony",                ret => Me.CurrentTarget != null && !Units.IsTargetWorthy(Me.CurrentTarget) || (Units.IsTargetWorthy(Me.CurrentTarget) && (Unit.TimeToDeath(Me.CurrentTarget) < 60 || Unit.TimeToDeath(Me.CurrentTarget) == 9999) && (!Buff.UnitsHasMyBuff("Bane of Doom") || !Buff.TargetHasDebuff("Bane of Doom"))), "Bane of Agony TTL=" + Unit.TimeToDeath(Me.CurrentTarget)),
                        Buff.CastDebuff("Corruption",                   ret => true, "Corruption"),
                        Spells.CastConicSpell("Shadowflame", 11f, 33f, ret => true, "ShadowFlame"),
                        Spells.CastSpell("Hand of Gul'dan",             ret => true, "Hand of Gul'dan"),
                        Spells.CastSelfSpell("Immolation Aura",         ret => Buff.PlayerBuffTimeLeft("Metamorphosis") > 10 && Unit.Instance.DistanceToTargetBoundingBox() <= 10f, "Immolation Aura"),
                        Spells.CastSpell("Incinerate",                  ret => Buff.PlayerHasActiveBuff("Molten Core"), "Incinerate"),
                        Spells.CastSpell("Shadow Bolt",                 ret => Buff.PlayerHasBuff("Shadow Trance"), "Instant Shadow Bolt"),
                        Spells.CastSpell("Soul Fire",                   ret => Buff.PlayerBuffTimeLeft("Decimation") > 0, "Soul Fire with Decimation"),
                        // Spells.CastSpell("Soul Fire",                ret => Buff.PlayerHasActiveBuff("Decimation"), "Soul Fire with Decimation"),
                        Spells.CastSelfSpell("Life Tap",                ret => Me.ManaPercent <= 30 && !Spell.PlayerIsChanneling && Me.HealthPercent > 40 && !Buff.UnitHasHasteBuff(Me) && !Buff.PlayerHasBuff("Metamorphosis") && !Buff.PlayerHasBuff("Demon Soul: Felguard"), "Life tap - mana < 30%"),
                        Spells.CastSpell("Incinerate",                  ret => true, "Incinerate"),
                        Spells.CastSelfSpell("Life Tap",                ret => Me.IsMoving && Me.HealthPercent > Me.ManaPercent && Me.ManaPercent < 80, "Life tap while moving"),
                        Spells.CastSpell("Fel Flame",                   ret => Me.IsMoving, "Fel flame while moving"),
                        Spells.CastSelfSpell("Life Tap",                ret => Me.ManaPercent < 100 && !Spell.PlayerIsChanneling && Me.HealthPercent > 40, "Life tap - mana < 100%"));
            }
        }

        public override Composite Medic
        {
            get
            {
                return new Decorator(
                    ret => Me.HealthPercent < 100 && SettingsFile.Instance.AutoManageHealing,
                    new PrioritySelector(Items.UseBagItem("Healthstone", ret => Me.HealthPercent < 40, "Healthstone")));
            }
        }

        public override Composite PreCombat
        {
            get
            {
                return new PrioritySelector(
                    new Decorator(
                        ret => !Me.Mounted && !Me.Dead && !Me.Combat && !Me.IsFlying && !Me.IsOnTransport && !Me.HasAura("Food") && !Me.HasAura("Drink"),
                        new PrioritySelector(
                            Buff.CastBuff("Fel Armor", ret => true, "Fel Armor"),
                            Pets.CastPetSummonSpell("Summon Felguard", ret => !Me.IsMoving && !Me.GotAlivePet, "Felguard"),
                            Buff.CastBuff("Soul Link", ret => Pet != null && Pet.IsAlive, "Soul Link"),
                            new Decorator(
                                ret => !Me.IsMoving,
                                new Sequence( // Waiting for a bit
                                            new ActionSleep(2000),
                                            Spells.ChannelSelfSpell("Soul Harvest", ret => Me.CurrentSoulShards < 2 && !Me.IsMoving, "[Shards] Soul Harvest - < 2 shards"))))));
            }
        }

        public override Composite Resting
        {
            get { return this.PreCombat; }
        }

        public override Composite PVPRotation
        {
            get { return this.SingleRotation; }
        }

        public override Composite PVERotation
        {
            get { return this.SingleRotation; }
        }    
    }
}
